package Impossible::Andrews;
use strict vars;
use Math::BigInt;
use Numeric::Choose;
use Numeric::IncreasingSequence;

$Impossible::Andrews::choose52_13=&::choose(52,13);
$Impossible::Andrews::choose39_13=&::choose(39,13);
$Impossible::Andrews::choose26_13=&::choose(26,13);


sub splitPageNo {
    my($pageNo,$explainer)=@_;
    my($start)=$pageNo;
    my($south)=$pageNo % $Impossible::Andrews::choose26_13;
    $pageNo = $pageNo / $Impossible::Andrews::choose26_13;
    my($east) = $pageNo % $Impossible::Andrews::choose39_13;
    my($north) = ($pageNo / $Impossible::Andrews::choose39_13) %
	$Impossible::Andrews::choose52_13;
    if ($explainer) {
        my($node)=$explainer->newSubStep("Split up $start into Hand Indices",
               "<div>$start = North*choose(39,13)*choose(26,13) + " .
			       "East*choose(26,13) + South</div>");
	$node->addResult("North=$north ; East=$east; South=$south");
    }

    return ($north,$east,$south);

}

sub new {
    my ($this)={};
    bless $this, shift;
}

sub decode {
    my ($this,$pageNum,$explainer)=@_;
    my (@nums)=&splitPageNo($pageNum,$explainer);
    my (@sequences)=();
    my (@hands)=qw(North East South);

    my ($substep);
    if ($explainer) {
	$substep=$explainer->newSubStep("Decode Hand Numbers into Hand Sequences");
    }

    my ($handno); 
    my (@results);
    foreach $handno (0..2) {
       	my(@seq)=&Numeric::IncreasingSequence::decode($nums[$handno],13,$substep,$hands[$handno]);
        push(@sequences,\@seq);
	if ($substep) {
	    push(@results,"<div>$hands[$handno]Sequence= [" . join(",",@seq)."]</div>");
	}
    }
    if ($substep) {
	$substep->addResult(join("",@results));
    }
    

    my (@locations)=0..51;
    my (@result);
    my ($handno);

    if ($explainer) {
	$substep=$explainer->newSubStep("Decode Hand Sequences into Hand Card Numbers");
    }

    for ($handno=0;$handno<3;$handno++) {
	my (@list)=@{$sequences[$handno]};
	my (@newlocs)=();
	my (@orig)=@list;
        my ($nextCard)=shift @list;
        my ($index)=0;
	my (@hand);

	foreach (@locations) {
	    if ($index == $nextCard) {
		$result[$_]=$handno;
		push(@hand,$_);
                $nextCard=shift @list;
	    } else {
		push(@newlocs,$_);
	    }
	    $index++;
	}

	if ($substep) {
	    my($text)="Out of:<div>[".join(", ",@locations)."]</div>".
		"take out the set of numbers corresponding to the indices in:".
		    "<div>".$hands[$handno]."Sequence = [" . join(",",@orig)."]</div>";
	    my($cardstep)=
		$substep->newSubStep("Decode ".$hands[$handno]."Sequence",$text);
	    my($res)="<div>".$hands[$handno]. "CardNumbers = [".join(",",@hand)."]</div>";
	    $cardstep->addResult($res);
	    $substep->addResult($res);
	    $cardstep->addResult("<div>Remainder = [" . join(", ",@newlocs) . "]</div>");
	}

	@locations=@newlocs;
    }
    foreach (@locations) { $result[$_]=3; }

    if ($substep) {
	my($text)="Place the rest with west.";
	my($cardstep)=$substep->newSubStep("Set West's Cards",$text);
	my($res)="<div>WestCardNumbers = [".join(",",@locations)."]</div>";
	$cardstep->addResult($res);
	$substep->addResult($res);
    }


    return @result;
}

1;
