#!/usr/bin/perl

use lib ".";

use Math::BigInt;
use Impossible::Andrews;
use Impossible::Pavlicek;
use Format::XML;
use Format::HTML;
use Format::Explainer;

use CGI;

$cgi=new CGI;

$baseBookName=$cgi->param("bookName");
if ($baseBookName ne "Pavlicek") {
    $baseBookName="Andrews";
}

if ($cgi->param("format") eq 'XML') {
    $formatter=new Format::XML($cgi);
} else {
    $formatter=new Format::HTML($cgi);
}

$book=eval "new Impossible::$baseBookName";

$scramble=($cgi->param("scramble") ne "");
if ($scramble) {
    $bookName="Scrambled $baseBookName";
} else {
    $bookName=$baseBookName;
}

$showWork=($cgi->param("showWork") ne "");
$formatter->formatStart($bookName);


@pageNumbers=split(" ",$cgi->param("pageNumbers"));

@cards=qw(A K Q J T 9 8 7 6 5 4 3 2);
@suitsymms=(
	'<font style="color:black">&spades;</font>',
	'<font style="color:red">&hearts;</font>',
	'<font style="color:red">&diams;</font>',
	'<font style="color:black">&clubs;</font>'
);

if ($scramble) { 
    $M=new Math::BigInt("13109994191499930367061460371");
    $N=new Math::BigInt("34563463456363563565356345634");
}
$D=new Math::BigInt("53644737765488792839237440000");

foreach $page (@pageNumbers) {

    if ($page!~/^[0-9]*[1-9][0-9]*$/) { next; }
    my($explainer,$substep);
    if ($showWork) {
	$explainer=new Format::Explainer("Show work for page number $page");
        my($max)=$D-1;
        $substep=$explainer->newSubStep("Get real page number in the unscrambled $baseBookName book", "Real Page Numbers are numbers between 0 and $max rather than 1 to $D");
    }

    $realpage=new Math::BigInt($page)-1;
    if ($scramble) {
        $realpage=($realpage*$M+$N);
    }
    realpage=$realpage % $D;
    if ($substep) {
        $substep->addResult("<div>RealPageNumber=$realpage</div>");
    }
    @dealseq=$book->decode($realpage,$explainer);
    
    @deal=(["","","",""],["","","",""],["","","",""],["","","",""]);

    for ($i=0;$i<52; $i++) {
	$handNo=$dealseq[$i];
        $suitNo=$i/13;
        $card=$cards[$i%13];
	$deal[$handNo]->[$suitNo] .= $card;
    }

    $formatter->formatDealWithExpl($page,$bookName,$explainer,@deal);
}

$formatter->formatEnd;
